#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <sys/select.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>

#define MAXLINE 256

#define SOCKET "./server.socket"

int main(int argc, char **argv)
{
  int sockfd, newsockfd, num_read, maxfd, status;
  struct sockaddr_un servaddr;
  char buffer[MAXLINE];
  fd_set base_set, set;
  struct timeval* timeout;

  /* creates socket stream domain unix */
  if ((sockfd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0){
    perror("server: can't open stream socket");
    exit(-1);
  }

  /* structure address cleaning */
  bzero(&servaddr, sizeof(servaddr));
  servaddr.sun_family = AF_UNIX;
  strcpy(servaddr.sun_path, SOCKET);

  /* if exists, deletes socket file name */
  unlink(servaddr.sun_path);

  /* binds socket to address */
  if (bind(sockfd, (struct sockaddr *) &servaddr, sizeof(servaddr)) < 0){
    perror("server, can't bind to address");
    exit(-1);
  }

  /* allows for 5 pending clients until connection is accepted */
  listen(sockfd, 5);
 
  maxfd = sockfd;
  
  /* setup the select mask */
  FD_ZERO(&base_set);
  FD_SET(sockfd, &base_set);

  while(1) {

    /* restore the programmed fd set */
    set = base_set;
    
    /* disable timeout */
    timeout = NULL;

    /*
     * 'select' supports timout unblocking in case there are no pending
     * requests in any of the programmed file descriptors.
     *
     * For example, to setup a timeout of 2 seconds do the following;
     *
     * struct timeval time = {2,0};
     * timeout = &time;
     * 
     */

     status = select(maxfd+1, &set, 0, 0, timeout);
     if (status < 0) {
	perror("server, error occurred.\n");
     } else if (status == 0) {
	printf("server, timeout.\n");
     } else {
	
        if (FD_ISSET(sockfd, &set)) {

           /* accept a connection on a socket */
	   newsockfd = accept(sockfd, (struct sockaddr *) NULL, NULL);
	    
	   /* read string from client */
	   num_read = read(newsockfd, buffer, MAXLINE);
	   if (num_read > 0) {
	      
	     /* echo string to client */
	     write(newsockfd, buffer, num_read);
	   }

	   close(newsockfd);
	   continue;
	}

	printf("server, wrong file descriptor.\n");
    }
  }
}
